@@ -18,6 +18,9 @@ module Fcf.Data.List
   , Snoc
   , Cons2
   , Init
+  , Uncons
+  , Unsnoc
+  , Singleton
   , Null
   , Length
 
@@ -40,6 +43,7 @@ module Fcf.Data.List
   -- * Sublists
   , Take
   , Drop
+  , SplitAt
   , TakeWhile
   , DropWhile
   , Span
@@ -81,10 +85,11 @@ import Fcf.Data.Nat
 import Fcf.Utils (If, TyEq)
 
 -- $setup
--- >>> :set -XGADTs -XUndecidableInstances
+-- >>> :set -XGADTs -XUndecidableInstances -XDataKinds
 -- >>> import Fcf.Core (Exp, Eval)
 -- >>> import Fcf.Combinators
 -- >>> import Fcf.Class.Foldable (Concat)
+-- >>> import Fcf.Class.Functor ()
 -- >>> import Fcf.Class.Monoid ()
 -- >>> import Fcf.Data.Nat
 -- >>> import Fcf.Utils (If, TyEq)
@@ -121,6 +126,36 @@ type instance Eval (Init (a ': '[])) = 'Just '[]
 type instance Eval (Init (a ': b ': as)) =
   Eval (Map (Cons a) =<< (Init (b ': as)))
 
+data Uncons :: [a] -> Exp (Maybe (a, [a]))
+type instance Eval (Uncons '[]) = 'Nothing
+type instance Eval (Uncons (a ': xs)) = 'Just '(a, xs)
+
+-- | Decompose a list into 'init' and 'last'
+--
+-- === __Example__
+--
+-- >>> :kind! Eval (Unsnoc '[])
+-- Eval (Unsnoc '[]) :: Maybe ([a], a)
+-- = Nothing
+--
+-- >>> :kind! Eval (Unsnoc '[1])
+-- Eval (Unsnoc '[1]) :: Maybe ([Natural], Natural)
+-- = Just '( '[], 1)
+--
+-- >>> :kind! Eval (Unsnoc '[1,2,3])
+-- Eval (Unsnoc '[1,2,3]) :: Maybe ([Natural], Natural)
+-- = Just '([1, 2], 3)
+data Unsnoc :: [a] -> Exp (Maybe ([a], a))
+type instance Eval (Unsnoc '[]) = 'Nothing
+type instance Eval (Unsnoc (x ': '[])) = 'Just '( '[], x)
+type instance Eval (Unsnoc (x ': y ': ys)) = Eval (Map (PrependF x) =<< Unsnoc (y ': ys))
+
+data PrependF :: a -> ([a], a) -> Exp ([a], a)
+type instance Eval (PrependF x '(xs, y)) = '(x ': xs, y)
+
+data Singleton :: a -> Exp [a]
+type instance Eval (Singleton x) = '[x]
+
 data Tail :: [a] -> Exp (Maybe [a])
 type instance Eval (Tail '[]) = 'Nothing
 type instance Eval (Tail (_a ': as)) = 'Just as
@@ -286,6 +321,17 @@ type family Drop_ (n :: Nat) (xs :: [a]) :: [a] where
   Drop_ _ '[]       = '[]
   Drop_ n (x ': xs) = Drop_ (n TL.- 1) xs
 
+-- | Return a tuple where first element is @xs@ prefix of length @n@
+-- and second element is the remainder of the list.
+--
+-- === __Example__
+--
+-- >>> :kind! Eval (SplitAt 3 '[1,2,3,4,5])
+-- Eval (SplitAt 3 '[1,2,3,4,5]) :: ([Natural], [Natural])
+-- = '([1, 2, 3], [4, 5])
+data SplitAt :: Nat -> [a] -> Exp ([a], [a])
+type instance Eval (SplitAt n xs) = '(Eval (Take n xs), Eval (Drop n xs))
+
 -- | Take the longest prefix of elements satisfying a predicate.
 --
 -- === __Example__
